@@ -1,4 +1,5 @@
 import os.path
+import subprocess
 
 from anasymod.generators.vivado import VivadoTCLGenerator
 from anasymod.generators.codegen import CodeGenerator
@@ -24,6 +25,22 @@ class VivadoEmulation(VivadoTCLGenerator):
         super().__init__(target=target)
 
     def build(self):
+        #subst drive
+        drive = 'V:'
+        # Check if on-chip memory is sufficient on selected FPGA board
+        if self.target.prj_cfg.board.bram is not None:
+            bits_per_sample = 0
+            probes = (self.target.str_cfg.digital_probes + self.target.str_cfg.analog_probes +
+                      [self.target.str_cfg.time_probe] + [self.target.str_cfg.dec_cmp])
+            for probe in probes:
+                bits_per_sample += int(probe.width)
+            if (bits_per_sample * self.target.prj_cfg.ila_depth) > (self.target.prj_cfg.board.bram * 8):
+                raise(f'ERROR: Number ob samples to be recorded does not fit on FPGA board, please either select a '
+                      f'board with more block memory, or change the ila_depth')
+        else:
+            print(f'WARNING: Check for sufficient BRAM could not be conducted, '
+                  f'not enough information given in board definition!')
+
         scfg = self.target.str_cfg
         """ type : StructureConfig """
         project_root = self.target.project_root
@@ -31,7 +48,8 @@ class VivadoEmulation(VivadoTCLGenerator):
         # subst command to substitute project directory to a drive letter
         if os.name == 'nt':
             if len(back2fwd(self.target.project_root)) > 80:
-                project_root = self.subst_path(drive='V:')
+
+                project_root = self.subst_path(drive=drive)
 
         # create a new project
         self.create_project(
@@ -89,6 +107,13 @@ class VivadoEmulation(VivadoTCLGenerator):
             # Setup Debug Hub
             constrs.use_templ(TemplDbgHub(target=self.target))
 
+            # Add false paths for Zynq control signals.  This is necessary in some cases
+            # to provide timing violations, since the ARM core is running on a different
+            # clock that the emulator circuitry.  This is a real problem, but is handled
+            # in firmware with handshaking and very short delays.
+            if self.target.cfg.fpga_sim_ctrl == FPGASimCtrl.UART_ZYNQ:
+                constrs.writeln('set_false_path -through [get_pins sim_ctrl_gen_i/zynq_gpio_i/*]')
+
             # write master constraints to file and add to project
             master_constr_path = os.path.join(self.target.prj_cfg.build_root, 'constrs.xdc')
             constrs.write_to_file(master_constr_path)
@@ -100,6 +125,12 @@ class VivadoEmulation(VivadoTCLGenerator):
             for file in xci_file.files:
                 self.writeln(f'read_ip "{back2fwd(file)}"')
 
+        # read user-provided TCL scripts
+        self.writeln('# Custom user-provided TCL scripts')
+        for tcl_file in self.target.content.tcl_files:
+            for file in tcl_file.files:
+                self.writeln(f'source "{back2fwd(file)}"')
+
         # upgrade IPs as necessary
         self.writeln('if {[get_ips] ne ""} {')
         self.writeln('    upgrade_ip [get_ips]')
@@ -108,10 +139,9 @@ class VivadoEmulation(VivadoTCLGenerator):
         # generate all IPs
         self.writeln('generate_target all [get_ips]')
 
-        # create block diagram if needed
-        # TODO: pass through configuration options
+        # create additional Hardware for control interface
         if self.target.cfg.fpga_sim_ctrl == FPGASimCtrl.UART_ZYNQ:
-            self.writeln(TemplZynqGPIO().text)
+            self.use_templ(TemplZynqGPIO(is_ultrascale=scfg.is_ultrascale))
 
         # launch the build and wait for it to finish
         num_cores = min(int(self.target.prj_cfg.vivado_config.num_cores), 8)
@@ -146,7 +176,21 @@ class VivadoEmulation(VivadoTCLGenerator):
                 self.writeln('exec subst ' + self.subst + ' ' + self.old_subst)
 
         # run bitstream generation
-        self.run(filename=r"bitstream.tcl")
+        ret_error = self.run(filename=r"bitstream.tcl", stack=self.target.prj_cfg.cfg.vivado_stack, return_error=True)
+        if os.name == 'nt':
+            if ret_error:
+                #remove and restore drive substitutions
+                if self.subst:
+                    try:
+                        subprocess.call(f'subst {drive} /d', shell=True)
+                    except:
+                        print(f'WARNING: Removing mapped drive:{drive} did not work.')
+                    if self.old_subst:
+                        try:
+                            subprocess.call(f'subst {drive} {self.old_subst}', shell=True)
+                        except:
+                            print(f'WARNING: Mapping of drive:{drive} to network path: {self.old_subst} did not work.')
+
 
     def run_FPGA(self, **kwargs):
         """
